@@ -34,6 +34,7 @@
 #include "plain/plain.h"
 #include "securid/securid.h"
 #include "digest-md5/digest-md5.h"
+#include "scram/scram.h"
 
 #include "login/login.h"
 #include "ntlm/x-ntlm.h"
@@ -53,6 +54,12 @@ register_builtin_mechs (Gsasl * ctx)
 {
   int rc = GSASL_OK;
 
+#ifdef USE_SCRAM_SHA1
+  rc = gsasl_register (ctx, &gsasl_scram_sha1_mechanism);
+  if (rc != GSASL_OK)
+    return rc;
+#endif /* USE_SCRAM_SHA1 */
+
 #ifdef USE_ANONYMOUS
   rc = gsasl_register (ctx, &gsasl_anonymous_mechanism);
   if (rc != GSASL_OK)
