@@ -130,6 +130,7 @@ greeting (void)
   return 1;
 }
 
+#ifdef HAVE_LIBGNUTLS
 static int
 has_starttls (void)
 {
@@ -151,6 +152,7 @@ starttls (void)
 
   return 1;
 }
+#endif
 
 static int
 select_mechanism (char **mechlist)
@@ -263,6 +265,9 @@ const char version_etc_copyright[] =
      year.  */
   "Copyright %s %d Simon Josefsson.";
 
+static void
+usage (int status) GSASL_ATTR_NO_RETRUN;
+
 static void
 usage (int status)
 {
@@ -304,7 +309,7 @@ main (int argc, char *argv[])
     wVersionRequested = MAKEWORD (2, 0);
     r = WSAStartup (wVersionRequested, &wsaData);
     if (r)
-      error (EXIT_FAILURE, 0, _("Cannot initialize Windows sockets."));
+      error (EXIT_FAILURE, 0, _("cannot initialize Windows sockets"));
   }
 #endif
 
@@ -327,9 +332,10 @@ main (int argc, char *argv[])
   if (!(args_info.client_flag || args_info.client_given) &&
       !args_info.server_given &&
       !args_info.client_mechanisms_flag && !args_info.server_mechanisms_flag)
-    error (EXIT_FAILURE, 0,
-	   _("missing argument\nTry `%s --help' for more information."),
-	   program_name);
+    {
+      error (0, 0, _("missing argument"));
+      usage (EXIT_FAILURE);
+    }
 
   if ((args_info.x509_cert_file_arg && !args_info.x509_key_file_arg) ||
       (!args_info.x509_cert_file_arg && args_info.x509_key_file_arg))
@@ -738,28 +744,30 @@ main (int argc, char *argv[])
 		  ssize_t len;
 
 		  len = getline (&line, &n, stdin);
-		  if (len < 0)
+		  if (len <= 0)
 		    break;
 
 		  if (args_info.imap_flag || args_info.smtp_flag)
 		    {
-		      int pos = strlen (line);
-
-		      if (pos < 2 || strcmp (&line[pos-2], "\r\n") != 0)
+		      if (len < 2 || strcmp (&line[len-2], "\r\n") != 0)
 			{
-			  char *tmp = realloc (line, pos + 1);
+			  char *tmp = realloc (line, len + 2);
 			  if (!tmp)
 			    error (EXIT_FAILURE, errno, "realloc");
 			  line = tmp;
-			  line[pos - 1] = '\r';
-			  line[pos] = '\n';
-			  line[pos + 1] = '\0';
+			  line[len - 1] = '\r';
+			  line[len] = '\n';
+			  line[len + 1] = '\0';
+			  len++;
 			}
 		    }
 		  else
-		    line[strlen (line) - 1] = '\0';
+		    {
+		      len--;
+		      line[len] = '\0';
+		    }
 
-		  res = gsasl_encode (xctx, line, strlen (line),
+		  res = gsasl_encode (xctx, line, len,
 				      &out, &output_len);
 		  if (res != GSASL_OK)
 		    break;
