@@ -56,11 +56,7 @@
 
 /* Get i18n. */
 #include <gettext.h>
-#ifdef HAVE_LOCALE_H
-# include <locale.h>
-#else
-# define setlocale(Category, Locale)	/* empty */
-#endif
+#include <locale.h>
 #define _(String) gettext (String)
 #define gettext_noop(String) String
 #define N_(String) gettext_noop (String)
@@ -71,4 +67,11 @@ extern int readln (char **out);
 #include "gsasl_cmd.h"
 extern struct gengetopt_args_info args_info;
 
+/* This feature is available in gcc versions 2.5 and later.  */
+#if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 5)
+# define GSASL_ATTR_NO_RETRUN
+#else
+# define GSASL_ATTR_NO_RETRUN __attribute__ ((__noreturn__))
+#endif
+
 #endif /* _INTERNAL_H */
